@@ -94,22 +94,31 @@ typedef struct {
 
 static int snd_pcm_pipewire_stop(snd_pcm_ioplug_t *io);
 
+static int block_check(snd_pcm_ioplug_t *io)
+{
+	snd_pcm_pipewire_t *pw = io->private_data;
+	snd_pcm_sframes_t avail;
+	uint64_t val;
+
+	avail = snd_pcm_ioplug_avail(io, pw->hw_ptr, io->appl_ptr);
+	if (avail >= 0 && avail < (snd_pcm_sframes_t)pw->min_avail) {
+		spa_system_eventfd_read(pw->system, io->poll_fd, &val);
+		return 1;
+	}
+	return 0;
+}
+
 static int pcm_poll_block_check(snd_pcm_ioplug_t *io)
 {
-	uint64_t val;
-	snd_pcm_sframes_t avail;
 	snd_pcm_pipewire_t *pw = io->private_data;
 
 	if (io->state == SND_PCM_STATE_DRAINING) {
+		uint64_t val;
 		spa_system_eventfd_read(pw->system, io->poll_fd, &val);
 		return 0;
 	} else if (io->state == SND_PCM_STATE_RUNNING ||
 			   (io->state == SND_PCM_STATE_PREPARED && io->stream == SND_PCM_STREAM_CAPTURE)) {
-		avail = snd_pcm_avail_update(io->pcm);
-		if (avail >= 0 && avail < (snd_pcm_sframes_t)pw->min_avail) {
-			spa_system_eventfd_read(pw->system, io->poll_fd, &val);
-			return 1;
-		}
+		return block_check(io);
 	}
 	return 0;
 }
@@ -117,8 +126,14 @@ static int pcm_poll_block_check(snd_pcm_ioplug_t *io)
 static inline int pcm_poll_unblock_check(snd_pcm_ioplug_t *io)
 {
 	snd_pcm_pipewire_t *pw = io->private_data;
-	spa_system_eventfd_write(pw->system, pw->fd, 1);
-	return 1;
+	snd_pcm_uframes_t avail;
+
+	avail = snd_pcm_ioplug_avail(io, pw->hw_ptr, io->appl_ptr);
+	if (avail >= pw->min_avail || io->state == SND_PCM_STATE_DRAINING) {
+		spa_system_eventfd_write(pw->system, pw->fd, 1);
+		return 1;
+	}
+	return 0;
 }
 
 static void snd_pcm_pipewire_free(snd_pcm_pipewire_t *pw)
@@ -147,6 +162,15 @@ static int snd_pcm_pipewire_close(snd_pcm_ioplug_t *io)
 	return 0;
 }
 
+static int snd_pcm_pipewire_poll_descriptors(snd_pcm_ioplug_t *io, struct pollfd *pfds, unsigned int space)
+{
+	snd_pcm_pipewire_t *pw = io->private_data;
+	pcm_poll_unblock_check(io); /* unblock socket for polling if needed */
+	pfds->fd = pw->fd;
+	pfds->events = POLLIN | POLLERR | POLLNVAL;
+	return 1;
+}
+
 static int snd_pcm_pipewire_poll_revents(snd_pcm_ioplug_t *io,
 				     struct pollfd *pfds, unsigned int nfds,
 				     unsigned short *revents)
@@ -344,6 +368,15 @@ static void on_stream_io_changed(void *data, uint32_t id, void *area, uint32_t s
 	}
 }
 
+static void on_stream_drained(void *data)
+{
+	snd_pcm_pipewire_t *pw = data;
+	pw->drained = true;
+	pw->draining = false;
+	pw_log_debug(NAME" %p: drained", pw);
+	pw_thread_loop_signal(pw->main_loop, false);
+}
+
 static void on_stream_process(void *data)
 {
 	snd_pcm_pipewire_t *pw = data;
@@ -372,22 +405,17 @@ static void on_stream_process(void *data)
 	pw_stream_queue_buffer(pw->stream, b);
 
 	if (io->state == SND_PCM_STATE_DRAINING && !pw->draining && hw_avail == 0) {
-		pw_stream_flush(pw->stream, true);
-		pw->draining = true;
-		pw->drained = false;
+		if (io->stream == SND_PCM_STREAM_CAPTURE) {
+			on_stream_drained (pw); /* since pw_stream does not call drained() for capture */
+		} else {
+			pw_stream_flush(pw->stream, true);
+			pw->draining = true;
+			pw->drained = false;
+		}
 	}
 	pcm_poll_unblock_check(io); /* unblock socket for polling if needed */
 }
 
-static void on_stream_drained(void *data)
-{
-	snd_pcm_pipewire_t *pw = data;
-	pw->drained = true;
-	pw->draining = false;
-	pw_log_debug(NAME" %p: drained", pw);
-	pw_thread_loop_signal(pw->main_loop, false);
-}
-
 static const struct pw_stream_events stream_events = {
 	PW_VERSION_STREAM_EVENTS,
 	.param_changed = on_stream_param_changed,
@@ -396,14 +424,25 @@ static const struct pw_stream_events stream_events = {
 	.drained = on_stream_drained,
 };
 
+static int pipewire_start(snd_pcm_pipewire_t *pw)
+{
+	if (!pw->activated && pw->stream != NULL) {
+		pw_stream_set_active(pw->stream, true);
+		pw->activated = true;
+	}
+	return 0;
+}
+
 static int snd_pcm_pipewire_drain(snd_pcm_ioplug_t *io)
 {
 	int res;
 	snd_pcm_pipewire_t *pw = io->private_data;
 
 	pw_thread_loop_lock(pw->main_loop);
+	pw_log_debug(NAME" %p: drain", pw);
 	pw->drained = false;
 	pw->draining = false;
+	pipewire_start(pw);
 	while (!pw->drained && pw->error >= 0 && pw->activated) {
 		pw_thread_loop_wait(pw->main_loop);
 	}
@@ -457,6 +496,7 @@ static int snd_pcm_pipewire_prepare(snd_pcm_ioplug_t *io)
 		goto error;
 
 	pw_properties_set(props, PW_KEY_CLIENT_API, "alsa");
+	pw_properties_setf(props, PW_KEY_APP_NAME, "%s", pw_get_prgname());
 
 	if (pw_properties_get(props, PW_KEY_NODE_LATENCY) == NULL)
 		pw_properties_setf(props, PW_KEY_NODE_LATENCY, "%lu/%u", pw->min_avail, io->rate);
@@ -510,10 +550,8 @@ static int snd_pcm_pipewire_start(snd_pcm_ioplug_t *io)
 
 	pw_thread_loop_lock(pw->main_loop);
 	pw_log_debug(NAME" %p:", pw);
-	if (!pw->activated && pw->stream != NULL) {
-		pw_stream_set_active(pw->stream, true);
-		pw->activated = true;
-	}
+	pipewire_start(pw);
+	block_check(io); /* unblock socket for polling if needed */
 	pw_thread_loop_unlock(pw->main_loop);
 	return 0;
 }
@@ -557,30 +595,19 @@ static int snd_pcm_pipewire_pause(snd_pcm_ioplug_t * io, int enable)
 static int set_default_channels(struct spa_audio_info_raw *info)
 {
 	switch (info->channels) {
-	case 7:
-		info->position[5] = SPA_AUDIO_CHANNEL_SL;
-		info->position[6] = SPA_AUDIO_CHANNEL_SR;
-		SPA_FALLTHROUGH
-	case 5:
-		info->position[3] = SPA_AUDIO_CHANNEL_RL;
-		info->position[4] = SPA_AUDIO_CHANNEL_RR;
-		info->position[2] = SPA_AUDIO_CHANNEL_FC;
-		info->position[0] = SPA_AUDIO_CHANNEL_FL;
-		info->position[1] = SPA_AUDIO_CHANNEL_FR;
-		return 1;
 	case 8:
 		info->position[6] = SPA_AUDIO_CHANNEL_SL;
 		info->position[7] = SPA_AUDIO_CHANNEL_SR;
 		SPA_FALLTHROUGH
 	case 6:
-		info->position[4] = SPA_AUDIO_CHANNEL_RL;
-		info->position[5] = SPA_AUDIO_CHANNEL_RR;
+		info->position[5] = SPA_AUDIO_CHANNEL_LFE;
+		SPA_FALLTHROUGH
+	case 5:
+		info->position[4] = SPA_AUDIO_CHANNEL_FC;
 		SPA_FALLTHROUGH
 	case 4:
-		info->position[3] = SPA_AUDIO_CHANNEL_LFE;
-		SPA_FALLTHROUGH
-	case 3:
-		info->position[2] = SPA_AUDIO_CHANNEL_FC;
+		info->position[2] = SPA_AUDIO_CHANNEL_RL;
+		info->position[3] = SPA_AUDIO_CHANNEL_RR;
 		SPA_FALLTHROUGH
 	case 2:
 		info->position[0] = SPA_AUDIO_CHANNEL_FL;
@@ -728,9 +755,24 @@ static enum snd_pcm_chmap_position channel_to_chmap(enum spa_audio_channel chann
 	return SND_CHMAP_UNKNOWN;
 }
 
+static enum spa_audio_channel chmap_to_channel(enum snd_pcm_chmap_position pos)
+{
+	if (pos < 0 || pos >= SPA_N_ELEMENTS(chmap_info))
+		return SPA_AUDIO_CHANNEL_UNKNOWN;
+	return chmap_info[pos].channel;
+}
+
 static int snd_pcm_pipewire_set_chmap(snd_pcm_ioplug_t * io,
 				const snd_pcm_chmap_t * map)
 {
+	snd_pcm_pipewire_t *pw = io->private_data;
+	unsigned int i;
+
+	pw->format.channels = map->channels;
+	for (i = 0; i < map->channels; i++) {
+		pw->format.position[i] = chmap_to_channel(map->pos[i]);
+		pw_log_debug("map %d: %d %d", i, map->pos[i], pw->format.position[i]);
+	}
 	return 1;
 }
 
@@ -767,17 +809,16 @@ static snd_pcm_chmap_query_t **snd_pcm_pipewire_query_chmaps(snd_pcm_ioplug_t *i
 {
 	snd_pcm_chmap_query_t **maps;
 
-	maps = calloc(9, sizeof(*maps));
+	maps = calloc(7, sizeof(*maps));
 	make_map(maps,  0, 1, SND_CHMAP_MONO);
 	make_map(maps,  1, 2, SND_CHMAP_FL, SND_CHMAP_FR);
-	make_map(maps,  2, 3, SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_FC);
-	make_map(maps,  3, 4, SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_FC, SND_CHMAP_LFE);
-	make_map(maps,  4, 5, SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_FC, SND_CHMAP_RL, SND_CHMAP_RR);
-	make_map(maps,  5, 6, SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_FC, SND_CHMAP_LFE, SND_CHMAP_RL, SND_CHMAP_RR);
-	make_map(maps,  6, 7, SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_FC,
-			SND_CHMAP_SL, SND_CHMAP_SR, SND_CHMAP_RL, SND_CHMAP_RR);
-	make_map(maps,  7, 8, SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_FC, SND_CHMAP_LFE,
-			SND_CHMAP_SL, SND_CHMAP_SR, SND_CHMAP_RL, SND_CHMAP_RR);
+	make_map(maps,  2, 4, SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_RL, SND_CHMAP_RR);
+	make_map(maps,  3, 5, SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_RL, SND_CHMAP_RR,
+			SND_CHMAP_FC);
+	make_map(maps,  4, 6, SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_RL, SND_CHMAP_RR,
+			SND_CHMAP_FC, SND_CHMAP_LFE);
+	make_map(maps,  5, 8, SND_CHMAP_FL, SND_CHMAP_FR, SND_CHMAP_RL, SND_CHMAP_RR,
+			SND_CHMAP_FC, SND_CHMAP_LFE, SND_CHMAP_SL, SND_CHMAP_SR);
 
 	return maps;
 }
@@ -791,6 +832,7 @@ static snd_pcm_ioplug_callback_t pipewire_pcm_callback = {
 	.delay = snd_pcm_pipewire_delay,
 	.drain = snd_pcm_pipewire_drain,
 	.prepare = snd_pcm_pipewire_prepare,
+	.poll_descriptors = snd_pcm_pipewire_poll_descriptors,
 	.poll_revents = snd_pcm_pipewire_poll_revents,
 	.hw_params = snd_pcm_pipewire_hw_params,
 	.set_chmap = snd_pcm_pipewire_set_chmap,
