@@ -272,6 +272,7 @@ spa_alsa_enum_format(struct state *state, int seq, uint32_t start, uint32_t num,
 	size_t i, j;
 	int err, dir;
 	unsigned int min, max;
+	unsigned int rrate, rchannels;
 	uint8_t buffer[4096];
 	struct spa_pod_builder b = { 0 };
 	struct spa_pod_choice *choice;
@@ -298,6 +299,25 @@ spa_alsa_enum_format(struct state *state, int seq, uint32_t start, uint32_t num,
 	snd_pcm_hw_params_alloca(&params);
 	CHECK(snd_pcm_hw_params_any(hndl, params), "Broken configuration: no configurations available");
 
+	CHECK(snd_pcm_hw_params_set_rate_resample(hndl, params, 0), "set_rate_resample");
+
+	if (state->default_channels != 0) {
+		rchannels = state->default_channels;
+		CHECK(snd_pcm_hw_params_set_channels_near(hndl, params, &rchannels), "set_channels");
+		if (state->default_channels != rchannels) {
+			spa_log_warn(state->log, NAME" %s: Channels doesn't match (requested %u, got %u)",
+				state->props.device, state->default_channels, rchannels);
+		}
+	}
+	if (state->default_rate != 0) {
+		rrate = state->default_rate;
+		CHECK(snd_pcm_hw_params_set_rate_near(hndl, params, &rrate, 0), "set_rate_near");
+		if (state->default_rate != rrate) {
+			spa_log_warn(state->log, NAME" %s: Rate doesn't match (requested %u, got %u)",
+				state->props.device, state->default_rate, rrate);
+		}
+	}
+
 	spa_pod_builder_push_object(&b, &f[0], SPA_TYPE_OBJECT_Format, SPA_PARAM_EnumFormat);
 	spa_pod_builder_add(&b,
 			SPA_FORMAT_mediaType,    SPA_POD_Id(SPA_MEDIA_TYPE_audio),
@@ -345,14 +365,16 @@ spa_alsa_enum_format(struct state *state, int seq, uint32_t start, uint32_t num,
 				offs += snprintf(&buf[offs], sizeof(buf) - offs,
 						"%s ", snd_pcm_format_name((snd_pcm_format_t)i));
 		}
-		spa_log_warn(state->log, "unsupported card: formats:%s", buf);
+		spa_log_warn(state->log, "%s: unsupported card: formats:%s",
+				state->props.device, buf);
 
 		for (i = 0, offs = 0; i <= SND_PCM_ACCESS_LAST; i++) {
 			if (snd_pcm_access_mask_test(amask, (snd_pcm_access_t)i))
 				offs += snprintf(&buf[offs], sizeof(buf) - offs,
 						"%s ", snd_pcm_access_name((snd_pcm_access_t)i));
 		}
-		spa_log_warn(state->log, "unsupported card: access:%s", buf);
+		spa_log_warn(state->log, "%s: unsupported card: access:%s",
+				state->props.device, buf);
 		return -ENOTSUP;
 	}
 	if (j > 1)
@@ -508,7 +530,8 @@ int spa_alsa_set_format(struct state *state, struct spa_audio_info *fmt, uint32_
 	/* get format info */
 	format = spa_format_to_alsa(info->format, &planar);
 	if (format == SND_PCM_FORMAT_UNKNOWN) {
-		spa_log_warn(state->log, NAME" %p: unknown format %u", state, info->format);
+		spa_log_warn(state->log, NAME" %s: unknown format %u",
+				state->props.device, info->format);
 		return -EINVAL;
 	}
 
@@ -530,8 +553,8 @@ int spa_alsa_set_format(struct state *state, struct spa_audio_info *fmt, uint32_
 		if ((err = snd_pcm_hw_params_set_access(hndl, params,
 				planar ? SND_PCM_ACCESS_RW_NONINTERLEAVED
 				: SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
-			spa_log_error(state->log, NAME" %p: RW not possible: %s", state,
-					snd_strerror(err));
+			spa_log_error(state->log, NAME" %s: RW not possible: %s",
+					state->props.device, snd_strerror(err));
 			return err;
 		}
 	}
@@ -551,8 +574,8 @@ int spa_alsa_set_format(struct state *state, struct spa_audio_info *fmt, uint32_
 	rchannels = info->channels;
 	CHECK(snd_pcm_hw_params_set_channels_near(hndl, params, &rchannels), "set_channels");
 	if (rchannels != info->channels) {
-		spa_log_warn(state->log, NAME" %p: Channels doesn't match (requested %u, get %u",
-				state, info->channels, rchannels);
+		spa_log_warn(state->log, NAME" %s: Channels doesn't match (requested %u, got %u)",
+				state->props.device, info->channels, rchannels);
 		if (!SPA_FLAG_IS_SET(flags, SPA_NODE_PARAM_FLAG_NEAREST))
 			return -EINVAL;
 		info->channels = rchannels;
@@ -563,8 +586,8 @@ int spa_alsa_set_format(struct state *state, struct spa_audio_info *fmt, uint32_
 	rrate = info->rate;
 	CHECK(snd_pcm_hw_params_set_rate_near(hndl, params, &rrate, 0), "set_rate_near");
 	if (rrate != info->rate) {
-		spa_log_warn(state->log, NAME" %p: Rate doesn't match (requested %iHz, get %iHz)",
-				state, info->rate, rrate);
+		spa_log_warn(state->log, NAME" %s: Rate doesn't match (requested %iHz, got %iHz)",
+				state->props.device, info->rate, rrate);
 		if (!SPA_FLAG_IS_SET(flags, SPA_NODE_PARAM_FLAG_NEAREST))
 			return -EINVAL;
 		info->rate = rrate;
@@ -678,8 +701,8 @@ int spa_alsa_silence(struct state *state, snd_pcm_uframes_t silence)
 		frames = state->buffer_frames;
 
 		if (SPA_UNLIKELY((res = snd_pcm_mmap_begin(hndl, &my_areas, &offset, &frames)) < 0)) {
-			spa_log_error(state->log, NAME" %p: snd_pcm_mmap_begin error: %s",
-					state, snd_strerror(res));
+			spa_log_error(state->log, NAME" %s: snd_pcm_mmap_begin error: %s",
+					state->props.device, snd_strerror(res));
 			return res;
 		}
 		silence = SPA_MIN(silence, frames);
@@ -688,8 +711,8 @@ int spa_alsa_silence(struct state *state, snd_pcm_uframes_t silence)
 		snd_pcm_areas_silence(my_areas, offset, state->channels, silence, state->format);
 
 		if (SPA_UNLIKELY((res = snd_pcm_mmap_commit(hndl, offset, silence)) < 0)) {
-			spa_log_error(state->log, NAME" %p: snd_pcm_mmap_commit error: %s",
-					state, snd_strerror(res));
+			spa_log_error(state->log, NAME" %s: snd_pcm_mmap_commit error: %s",
+					state->props.device, snd_strerror(res));
 			return res;
 		}
 	} else {
@@ -714,8 +737,8 @@ static inline int do_start(struct state *state)
 	if (SPA_UNLIKELY(!state->alsa_started)) {
 		spa_log_trace(state->log, NAME" %p: snd_pcm_start", state);
 		if ((res = snd_pcm_start(state->hndl)) < 0) {
-			spa_log_error(state->log, NAME" %p: snd_pcm_start: %s",
-					state, snd_strerror(res));
+			spa_log_error(state->log, NAME" %s: snd_pcm_start: %s",
+					state->props.device, snd_strerror(res));
 			return res;
 		}
 		state->alsa_started = true;
@@ -730,8 +753,8 @@ static int alsa_recover(struct state *state, int err)
 
 	snd_pcm_status_alloca(&status);
 	if (SPA_UNLIKELY((res = snd_pcm_status(state->hndl, status)) < 0)) {
-		spa_log_error(state->log, NAME" %p: snd_pcm_status error: %s",
-				state, snd_strerror(res));
+		spa_log_error(state->log, NAME" %s: snd_pcm_status error: %s",
+				state->props.device, snd_strerror(res));
 		goto recover;
 	}
 
@@ -749,8 +772,8 @@ static int alsa_recover(struct state *state, int err)
 		delay = SPA_TIMEVAL_TO_USEC(&diff);
 		missing = delay * state->rate / SPA_USEC_PER_SEC;
 
-		spa_log_trace(state->log, NAME" %p: xrun of %"PRIu64" usec %"PRIu64" %f",
-				state, delay, missing, state->safety);
+		spa_log_trace(state->log, NAME" %p: xrun of %"PRIu64" usec %"PRIu64,
+				state, delay, missing);
 
 		spa_node_call_xrun(&state->callbacks,
 				SPA_TIMEVAL_TO_USEC(&trigger), delay, NULL);
@@ -759,20 +782,20 @@ static int alsa_recover(struct state *state, int err)
 		break;
 	}
 	case SND_PCM_STATE_SUSPENDED:
-		spa_log_info(state->log, NAME" %p: recover from state %s",
-				state, snd_pcm_state_name(st));
+		spa_log_info(state->log, NAME" %s: recover from state %s",
+				state->props.device, snd_pcm_state_name(st));
 		err = -ESTRPIPE;
 		break;
 	default:
-		spa_log_error(state->log, NAME" %p: recover from error state %s",
-				state, snd_pcm_state_name(st));
+		spa_log_error(state->log, NAME" %s: recover from error state %s",
+				state->props.device, snd_pcm_state_name(st));
 		break;
 	}
 
 recover:
 	if (SPA_UNLIKELY((res = snd_pcm_recover(state->hndl, err, true)) < 0)) {
-		spa_log_error(state->log, NAME" %p: snd_pcm_recover error: %s",
-				state, snd_strerror(res));
+		spa_log_error(state->log, NAME" %s: snd_pcm_recover error: %s",
+				state->props.device, snd_strerror(res));
 		return res;
 	}
 	spa_dll_init(&state->dll);
@@ -780,7 +803,7 @@ recover:
 	state->alsa_started = false;
 
 	if (state->stream == SND_PCM_STREAM_PLAYBACK)
-		spa_alsa_silence(state, state->threshold * 2);
+		spa_alsa_silence(state, state->threshold * 2 + state->headroom);
 
 	return do_start(state);
 }
@@ -794,8 +817,8 @@ static int get_status(struct state *state, snd_pcm_uframes_t *delay, snd_pcm_ufr
 		if ((res = alsa_recover(state, avail)) < 0)
 			return res;
 		if ((avail = snd_pcm_avail(state->hndl)) < 0) {
-			spa_log_warn(state->log, NAME" %p: snd_pcm_avail after recover: %s",
-					state, snd_strerror(avail));
+			spa_log_warn(state->log, NAME" %s: snd_pcm_avail after recover: %s",
+					state->props.device, snd_strerror(avail));
 			avail = state->threshold * 2;
 		}
 	} else {
@@ -805,20 +828,18 @@ static int get_status(struct state *state, snd_pcm_uframes_t *delay, snd_pcm_ufr
 
 	*target = state->threshold + state->headroom;
 
-#define MARGIN 48
 	if (state->resample && state->rate_match) {
 		state->delay = state->rate_match->delay * 2;
 		state->read_size = state->rate_match->size;
 		/* We try to compensate for the latency introduced by rate matching
-		 * by moving a little closer to the device read/write pointers.
-		 * Don't try to get closer than 48 samples but instead increase the
-		 * reported latency on the port (TODO). */
-		if (*target <= state->delay + MARGIN)
-			*target -= SPA_MAX(0, (int)(*target - MARGIN - state->delay));
+		 * by moving a little closer to the device read/write pointers. */
+		if (*target <= state->delay)
+			*target -= SPA_MAX(0, (int)(*target - state->delay));
 		else
 			*target -= state->delay;
 	} else {
-		state->delay = state->read_size = 0;
+		state->delay = 0;
+		state->read_size = state->threshold;
 	}
 
 	if (state->stream == SND_PCM_STREAM_PLAYBACK) {
@@ -828,7 +849,6 @@ static int get_status(struct state *state, snd_pcm_uframes_t *delay, snd_pcm_ufr
 		*delay = avail;
 		*target = SPA_MAX(*target, state->read_size);
 	}
-
 	return 0;
 }
 
@@ -840,7 +860,9 @@ static int update_time(struct state *state, uint64_t nsec, snd_pcm_sframes_t del
 	if (state->stream == SND_PCM_STREAM_PLAYBACK)
 		err = delay - target;
 	else
-		err = (target + 128) - delay;
+		err = target - delay;
+
+	err = SPA_CLAMP(err, -state->max_error, state->max_error);
 
 	if (SPA_UNLIKELY(state->dll.bw == 0.0)) {
 		spa_dll_set_bw(&state->dll, SPA_DLL_BW_MAX, state->threshold, state->rate);
@@ -851,16 +873,15 @@ static int update_time(struct state *state, uint64_t nsec, snd_pcm_sframes_t del
 
 	if (SPA_UNLIKELY(state->last_threshold != state->threshold)) {
 		int32_t diff = (int32_t) (state->last_threshold - state->threshold);
-		spa_log_trace(state->log, NAME" %p: follower:%d quantum change %d",
-				state, follower, diff);
-		state->next_time += diff / corr * 1e9 / state->rate;
+		spa_log_trace(state->log, NAME" %p: follower:%d quantum change %d -> %d (%d)",
+				state, follower, state->last_threshold, state->threshold, diff);
+		if (diff < 0)
+			state->next_time += diff / corr * 1e9 / state->rate;
 		state->last_threshold = state->threshold;
 	}
 
 	if (SPA_UNLIKELY((state->next_time - state->base_time) > BW_PERIOD)) {
 		state->base_time = state->next_time;
-		if (state->dll.bw > SPA_DLL_BW_MIN)
-			spa_dll_set_bw(&state->dll, state->dll.bw / 2.0, state->threshold, state->rate);
 
 		spa_log_debug(state->log, NAME" %p: follower:%d match:%d rate:%f "
 				"bw:%f thr:%d del:%ld target:%ld err:%f (%f %f %f)",
@@ -871,9 +892,9 @@ static int update_time(struct state *state, uint64_t nsec, snd_pcm_sframes_t del
 
 	if (state->rate_match) {
 		if (state->stream == SND_PCM_STREAM_PLAYBACK)
-			state->rate_match->rate = SPA_CLAMP(corr, 0.95, 1.05);
+			state->rate_match->rate = corr;
 		else
-			state->rate_match->rate = SPA_CLAMP(1.0/corr, 0.95, 1.05);
+			state->rate_match->rate = 1.0/corr;
 
 		SPA_FLAG_UPDATE(state->rate_match->flags, SPA_IO_RATE_MATCH_FLAG_ACTIVE, state->matching);
 	}
@@ -917,8 +938,9 @@ int spa_alsa_write(struct state *state)
 			return res;
 
 		if (SPA_UNLIKELY(!state->alsa_recovering && delay > target + state->threshold)) {
-			spa_log_warn(state->log, NAME" %p: follower delay:%ld resync %f %f %f",
-					state, delay, state->dll.z1, state->dll.z2, state->dll.z3);
+			spa_log_warn(state->log, NAME" %s: follower delay:%ld resync %f %f %f",
+					state->props.device, delay, state->dll.z1,
+					state->dll.z2, state->dll.z3);
 			spa_dll_init(&state->dll);
 			state->alsa_sync = true;
 		}
@@ -943,8 +965,8 @@ again:
 	frames = state->buffer_frames;
 	if (state->use_mmap) {
 		if (SPA_UNLIKELY((res = snd_pcm_mmap_begin(hndl, &my_areas, &offset, &frames)) < 0)) {
-			spa_log_error(state->log, NAME" %p: snd_pcm_mmap_begin error: %s",
-					state, snd_strerror(res));
+			spa_log_error(state->log, NAME" %s: snd_pcm_mmap_begin error: %s",
+					state->props.device, snd_strerror(res));
 			return res;
 		}
 		spa_log_trace_fp(state->log, NAME" %p: begin %ld %ld %d",
@@ -1033,15 +1055,15 @@ again:
 
 	if (state->use_mmap) {
 		if (SPA_UNLIKELY((commitres = snd_pcm_mmap_commit(hndl, offset, written)) < 0)) {
-			spa_log_error(state->log, NAME" %p: snd_pcm_mmap_commit error: %s",
-					state, snd_strerror(commitres));
+			spa_log_error(state->log, NAME" %s: snd_pcm_mmap_commit error: %s",
+					state->props.device, snd_strerror(commitres));
 			if (commitres != -EPIPE && commitres != -ESTRPIPE)
 				return res;
 		}
 
 		if (commitres > 0 && written != (snd_pcm_uframes_t) commitres) {
-			spa_log_warn(state->log, NAME" %p: mmap_commit wrote %ld instead of %ld",
-				     state, commitres, written);
+			spa_log_warn(state->log, NAME" %s: mmap_commit wrote %ld instead of %ld",
+				     state->props.device, commitres, written);
 		}
 	}
 
@@ -1076,9 +1098,9 @@ push_frames(struct state *state,
 	snd_pcm_uframes_t total_frames = 0;
 
 	if (spa_list_is_empty(&state->free)) {
-		spa_log_warn(state->log, NAME" %p: no more buffers", state);
+		spa_log_warn(state->log, NAME" %s: no more buffers", state->props.device);
 		total_frames = frames;
-	} else if (frames > 0) {
+	} else {
 		uint8_t *src;
 		size_t n_bytes, left;
 		struct buffer *b;
@@ -1154,8 +1176,9 @@ int spa_alsa_read(struct state *state, snd_pcm_uframes_t silence)
 			position = state->position->clock.position;
 			if (state->last_position && state->last_position + state->last_duration != position) {
 				state->alsa_sync = true;
-				spa_log_info(state->log, NAME" %p: discont, resync %"PRIu64" %"PRIu64" %d",
-						state, state->last_position, position, state->last_duration);
+				spa_log_info(state->log, NAME" %s: discont, resync %"PRIu64" %"PRIu64" %d",
+						state->props.device, state->last_position,
+						position, state->last_duration);
 			}
 			state->last_position = position;
 			state->last_duration = state->duration;
@@ -1170,17 +1193,18 @@ int spa_alsa_read(struct state *state, snd_pcm_uframes_t silence)
 		if ((res = get_status(state, &delay, &target)) < 0)
 			return res;
 
-		if (!state->alsa_recovering && (delay < target || delay > target * 2)) {
-			spa_log_warn(state->log, NAME" %p: follower delay:%lu target:%lu resync %f %f %f",
-					state, delay, target, state->dll.z1, state->dll.z2, state->dll.z3);
+		if (!state->alsa_recovering && (delay < target / 2 || delay > target * 2)) {
+			spa_log_warn(state->log, NAME" %s: follower delay:%lu target:%lu resync %f %f %f",
+					state->props.device, delay, target, state->dll.z1,
+					state->dll.z2, state->dll.z3);
 			spa_dll_init(&state->dll);
 			state->alsa_sync = true;
 		}
 		if (state->alsa_sync) {
-			spa_log_warn(state->log, NAME" %p: follower resync %ld %d %ld",
-					state, delay, threshold, target);
+			spa_log_warn(state->log, NAME" %s: follower resync %ld %d %ld",
+					state->props.device, delay, threshold, target);
 			if (delay < target)
-				snd_pcm_rewind(state->hndl, target - delay + 32);
+				snd_pcm_rewind(state->hndl, target - delay);
 			else if (delay > target)
 				snd_pcm_forward(state->hndl, delay - target);
 
@@ -1194,14 +1218,12 @@ int spa_alsa_read(struct state *state, snd_pcm_uframes_t silence)
 	}
 
 	frames = state->read_size;
-	if (frames == 0)
-		frames = state->threshold + state->delay;
 
 	if (state->use_mmap) {
 		to_read = state->buffer_frames;
 		if ((res = snd_pcm_mmap_begin(hndl, &my_areas, &offset, &to_read)) < 0) {
-			spa_log_error(state->log, NAME" %p: snd_pcm_mmap_begin error: %s",
-					state, snd_strerror(res));
+			spa_log_error(state->log, NAME" %s: snd_pcm_mmap_begin error: %s",
+					state->props.device, snd_strerror(res));
 			return res;
 		}
 		spa_log_trace_fp(state->log, NAME" %p: begin offs:%ld frames:%ld to_read:%ld thres:%d", state,
@@ -1219,15 +1241,15 @@ int spa_alsa_read(struct state *state, snd_pcm_uframes_t silence)
 		spa_log_trace_fp(state->log, NAME" %p: commit offs:%ld read:%ld count:%"PRIi64, state,
 				offset, read, state->sample_count);
 		if ((commitres = snd_pcm_mmap_commit(hndl, offset, read)) < 0) {
-			spa_log_error(state->log, NAME" %p: snd_pcm_mmap_commit error: %s",
-					state, snd_strerror(commitres));
+			spa_log_error(state->log, NAME" %s: snd_pcm_mmap_commit error: %s",
+					state->props.device, snd_strerror(commitres));
 			if (commitres != -EPIPE && commitres != -ESTRPIPE)
 				return res;
 		}
 
 		if (commitres > 0 && read != (snd_pcm_uframes_t) commitres) {
-			spa_log_warn(state->log, NAME" %p: mmap_commit read %ld instead of %ld",
-				     state, commitres, read);
+			spa_log_warn(state->log, NAME" %s: mmap_commit read %ld instead of %ld",
+				     state->props.device, commitres, read);
 		}
 	}
 
@@ -1241,7 +1263,7 @@ static int handle_play(struct state *state, uint64_t nsec,
 {
 	int res;
 
-	if (SPA_UNLIKELY(delay > target + state->last_threshold)) {
+	if (SPA_UNLIKELY(delay > target + state->threshold)) {
 		spa_log_trace(state->log, NAME" %p: early wakeup %ld %ld", state, delay, target);
 		state->next_time = nsec + (delay - target) * SPA_NSEC_PER_SEC / state->rate;
 		return -EAGAIN;
@@ -1271,14 +1293,14 @@ static int handle_capture(struct state *state, uint64_t nsec,
 	int res;
 	struct spa_io_buffers *io;
 
-	if (delay < target) {
+	if (SPA_UNLIKELY(delay < target)) {
 		spa_log_trace(state->log, NAME" %p: early wakeup %ld %ld", state, delay, target);
 		state->next_time = nsec + (target - delay) * SPA_NSEC_PER_SEC /
 			state->rate;
-		return 0;
+		return -EAGAIN;
 	}
 
-	if ((res = update_time(state, nsec, delay, target, false)) < 0)
+	if (SPA_UNLIKELY(res = update_time(state, nsec, delay, target, false)) < 0)
 		return res;
 
 	if ((res = spa_alsa_read(state, target)) < 0)
@@ -1384,6 +1406,24 @@ static inline bool is_following(struct state *state)
 	return state->position && state->clock && state->position->clock.id != state->clock->id;
 }
 
+static int setup_matching(struct state *state)
+{
+	int card;
+
+	state->matching = state->following;
+
+	if (state->position == NULL)
+		return -ENOTSUP;
+
+	spa_log_debug(state->log, "clock:%s card:%d", state->position->clock.name, state->card);
+	if (sscanf(state->position->clock.name, "api.alsa.%d", &card) == 1 &&
+	    card == state->card) {
+		state->matching = false;
+	}
+	state->resample = (state->rate != state->rate_denom) || state->matching;
+	return 0;
+}
+
 int spa_alsa_start(struct state *state)
 {
 	int err;
@@ -1391,30 +1431,25 @@ int spa_alsa_start(struct state *state)
 	if (state->started)
 		return 0;
 
-	state->following = is_following(state);
-	state->matching = state->following;
-
 	if (state->position) {
-		int card;
-		if (sscanf(state->position->clock.name, "api.alsa.%d", &card) == 1 &&
-		    card == state->card) {
-			state->matching = false;
-		}
 		state->duration = state->position->clock.duration;
 		state->rate_denom = state->position->clock.rate.denom;
 	}
 	else {
-		spa_log_warn(state->log, NAME" %p: no position set, using defaults", state);
+		spa_log_warn(state->log, NAME" %s: no position set, using defaults",
+				state->props.device);
 		state->duration = state->props.min_latency;
 		state->rate_denom = state->rate;
 	}
 
-	state->resample = (state->rate != state->rate_denom) || state->matching;
+	state->following = is_following(state);
+	setup_matching(state);
+
 	state->threshold = (state->duration * state->rate + state->rate_denom-1) / state->rate_denom;
 	state->last_threshold = state->threshold;
 
 	spa_dll_init(&state->dll);
-	state->safety = 0.0;
+	state->max_error = (256.0 * state->rate) / state->rate_denom;
 
 	spa_log_debug(state->log, NAME" %p: start %d duration:%d rate:%d follower:%d match:%d resample:%d",
 			state, state->threshold, state->duration, state->rate_denom,
@@ -1425,8 +1460,8 @@ int spa_alsa_start(struct state *state)
 		snd_pcm_dump(state->hndl, state->output);
 
 	if ((err = snd_pcm_prepare(state->hndl)) < 0 && err != -EBUSY) {
-		spa_log_error(state->log, NAME" %p: snd_pcm_prepare error: %s", state,
-				snd_strerror(err));
+		spa_log_error(state->log, NAME" %s: snd_pcm_prepare error: %s",
+				state->props.device, snd_strerror(err));
 		return err;
 	}
 
@@ -1443,7 +1478,7 @@ int spa_alsa_start(struct state *state)
 	state->alsa_started = false;
 
 	if (state->stream == SND_PCM_STREAM_PLAYBACK)
-		spa_alsa_silence(state, state->threshold * 2);
+		spa_alsa_silence(state, state->threshold * 2 + state->headroom);
 
 	if ((err = do_start(state)) < 0)
 		return err;
@@ -1481,6 +1516,7 @@ int spa_alsa_reassign_follower(struct state *state)
 		state->following = following;
 		spa_loop_invoke(state->data_loop, do_reassign_follower, 0, NULL, 0, true, state);
 	}
+	setup_matching(state);
 	return 0;
 }
 
@@ -1516,7 +1552,7 @@ int spa_alsa_pause(struct state *state)
 	spa_loop_invoke(state->data_loop, do_remove_source, 0, NULL, 0, true, state);
 
 	if ((err = snd_pcm_drop(state->hndl)) < 0)
-		spa_log_error(state->log, NAME" %p: snd_pcm_drop %s", state,
+		spa_log_error(state->log, NAME" %s: snd_pcm_drop %s", state->props.device,
 				snd_strerror(err));
 
 	state->started = false;
