@@ -76,6 +76,12 @@ struct impl {
 
 	uint64_t info_all;
 	struct spa_node_info info;
+#define IDX_EnumFormat		0
+#define IDX_PropInfo		1
+#define IDX_Props		2
+#define IDX_Format		3
+#define IDX_EnumPortConfig	4
+#define IDX_PortConfig		5
 	struct spa_param_info params[6];
 
 	struct spa_hook_list hooks;
@@ -206,9 +212,19 @@ static int link_io(struct impl *this)
 
 static void emit_node_info(struct impl *this, bool full)
 {
+	uint32_t i;
+
 	if (full)
 		this->info.change_mask = this->info_all;
 	if (this->info.change_mask) {
+		if (this->info.change_mask & SPA_NODE_CHANGE_MASK_PARAMS) {
+			for (i = 0; i < SPA_N_ELEMENTS(this->params); i++) {
+				if (this->params[i].user > 0) {
+					this->params[i].flags ^= SPA_PARAM_INFO_SERIAL;
+					this->params[i].user = 0;
+				}
+			}
+		}
 		spa_node_emit_info(&this->hooks, &this->info);
 		this->info.change_mask = 0;
 	}
@@ -585,15 +601,18 @@ static void convert_node_info(void *data, const struct spa_node_info *info)
 
 			switch (info->params[i].id) {
 			case SPA_PARAM_PropInfo:
-				idx = 1;
+				idx = IDX_PropInfo;
 				break;
 			case SPA_PARAM_Props:
-				idx = 2;
+				idx = IDX_Props;
 				break;
 			}
 			if (idx != SPA_ID_INVALID) {
-				this->params[idx] = info->params[i];
 				this->info.change_mask |= SPA_NODE_CHANGE_MASK_PARAMS;
+				this->params[idx].flags =
+					(this->params[idx].flags & SPA_PARAM_INFO_SERIAL) |
+					(info->params[i].flags & SPA_PARAM_INFO_READWRITE);
+				this->params[idx].user++;
 			}
 		}
 	}
@@ -662,12 +681,15 @@ static void follower_info(void *data, const struct spa_node_info *info)
 
 			switch (info->params[i].id) {
 			case SPA_PARAM_Props:
-				idx = 2;
+				idx = IDX_Props;
 				break;
 			}
 			if (idx != SPA_ID_INVALID) {
-				this->params[idx] = info->params[i];
 				this->info.change_mask |= SPA_NODE_CHANGE_MASK_PARAMS;
+				this->params[idx].flags =
+					(this->params[idx].flags & SPA_PARAM_INFO_SERIAL) |
+					(info->params[i].flags & SPA_PARAM_INFO_READWRITE);
+				this->params[idx].user++;
 			}
 		}
 	}
@@ -687,12 +709,15 @@ static void follower_port_info(void *data,
 
 			switch (info->params[i].id) {
 			case SPA_PARAM_Format:
-				idx = 3;
+				idx = IDX_Format;
 				break;
 			}
 			if (idx != SPA_ID_INVALID) {
-				this->params[idx] = info->params[i];
 				this->info.change_mask |= SPA_NODE_CHANGE_MASK_PARAMS;
+				this->params[idx].flags =
+					(this->params[idx].flags & SPA_PARAM_INFO_SERIAL) |
+					(info->params[i].flags & SPA_PARAM_INFO_READWRITE);
+				this->params[idx].user++;
 			}
 		}
 	}
@@ -1166,12 +1191,12 @@ impl_init(const struct spa_handle_factory *factory,
 		SPA_NODE_FLAG_IN_PORT_CONFIG |
 		SPA_NODE_FLAG_OUT_PORT_CONFIG |
 		SPA_NODE_FLAG_NEED_CONFIGURE;
-	this->params[0] = SPA_PARAM_INFO(SPA_PARAM_EnumFormat, SPA_PARAM_INFO_READ);
-	this->params[1] = SPA_PARAM_INFO(SPA_PARAM_PropInfo, SPA_PARAM_INFO_READ);
-	this->params[2] = SPA_PARAM_INFO(SPA_PARAM_Props, SPA_PARAM_INFO_READWRITE);
-	this->params[3] = SPA_PARAM_INFO(SPA_PARAM_Format, SPA_PARAM_INFO_WRITE);
-	this->params[4] = SPA_PARAM_INFO(SPA_PARAM_EnumPortConfig, SPA_PARAM_INFO_READ);
-	this->params[5] = SPA_PARAM_INFO(SPA_PARAM_PortConfig, SPA_PARAM_INFO_READWRITE);
+	this->params[IDX_EnumFormat] = SPA_PARAM_INFO(SPA_PARAM_EnumFormat, SPA_PARAM_INFO_READ);
+	this->params[IDX_PropInfo] = SPA_PARAM_INFO(SPA_PARAM_PropInfo, SPA_PARAM_INFO_READ);
+	this->params[IDX_Props] = SPA_PARAM_INFO(SPA_PARAM_Props, SPA_PARAM_INFO_READWRITE);
+	this->params[IDX_Format] = SPA_PARAM_INFO(SPA_PARAM_Format, SPA_PARAM_INFO_WRITE);
+	this->params[IDX_EnumPortConfig] = SPA_PARAM_INFO(SPA_PARAM_EnumPortConfig, SPA_PARAM_INFO_READ);
+	this->params[IDX_PortConfig] = SPA_PARAM_INFO(SPA_PARAM_PortConfig, SPA_PARAM_INFO_READWRITE);
 	this->info.params = this->params;
 	this->info.n_params = 6;
 
