@@ -72,6 +72,7 @@ struct node {
 	unsigned int active:1;
 	unsigned int mute:1;
 	uint32_t n_channels;
+	int64_t latency_offset;
 	uint32_t channels[SPA_AUDIO_MAX_CHANNELS];
 	float volumes[SPA_AUDIO_MAX_CHANNELS];
 };
@@ -447,6 +448,8 @@ static uint32_t profile_direction_mask(struct impl *this, uint32_t index)
 			have_output = true;
 		else if (profile == SPA_BT_PROFILE_A2DP_SOURCE)
 			have_input = true;
+		else if (profile == 0)
+			have_output = have_input = false;
 		else
 			have_output = have_input = true;
 		break;
@@ -624,13 +627,6 @@ static struct spa_pod *build_profile(struct impl *this, struct spa_pod_builder *
 		SPA_PARAM_PROFILE_description, SPA_POD_String(desc),
 		SPA_PARAM_PROFILE_available, SPA_POD_Id(SPA_PARAM_AVAILABILITY_yes),
 		0);
-	spa_pod_builder_prop(b, SPA_PARAM_ROUTE_devices, 0);
-	spa_pod_builder_push_array(b, &f[1]);
-	if (n_source > 0)
-		spa_pod_builder_int(b, capture[0]);
-	if (n_sink > 0)
-		spa_pod_builder_int(b, playback[0]);
-	spa_pod_builder_pop(b, &f[1]);
 	if (n_source > 0 || n_sink > 0) {
 		spa_pod_builder_prop(b, SPA_PARAM_PROFILE_classes, 0);
 		spa_pod_builder_push_struct(b, &f[1]);
@@ -666,8 +662,9 @@ static struct spa_pod *build_route(struct impl *this, struct spa_pod_builder *b,
 	const char *name_prefix, *description, *port_type;
 	enum spa_param_availability available;
 	enum spa_bt_form_factor ff;
+	const struct a2dp_codec *codec;
 	char name[128];
-	uint32_t i, mask;
+	uint32_t i, j, mask;
 
 	ff = spa_bt_form_factor_from_class(device->bluetooth_class);
 
@@ -769,10 +766,9 @@ static struct spa_pod *build_route(struct impl *this, struct spa_pod_builder *b,
 	spa_pod_builder_pop(b, &f[1]);
 	spa_pod_builder_prop(b, SPA_PARAM_ROUTE_profiles, 0);
 	spa_pod_builder_push_array(b, &f[1]);
-	for (i = 1; i < 3; i++) {
-		if (profile_direction_mask(this, i) & (1 << direction))
+	for (i = 1; (j = get_profile_from_index(this, i, &codec)) != SPA_ID_INVALID; i++)
+		if (profile_direction_mask(this, j) & (1 << direction))
 			spa_pod_builder_int(b, i);
-	}
 	spa_pod_builder_pop(b, &f[1]);
 
 	if (dev != SPA_ID_INVALID) {
@@ -795,6 +791,11 @@ static struct spa_pod *build_route(struct impl *this, struct spa_pod_builder *b,
 		spa_pod_builder_array(b, sizeof(uint32_t), SPA_TYPE_Id,
 				node->n_channels, node->channels);
 
+		if (this->profile == 1 && dev == DEVICE_ID_SINK) {
+			spa_pod_builder_prop(b, SPA_PROP_latencyOffsetNsec, 0);
+			spa_pod_builder_long(b, node->latency_offset);
+		}
+
 		spa_pod_builder_pop(b, &f[1]);
 	}
 
@@ -917,11 +918,19 @@ static int node_set_volume(struct impl *this, struct node *node, float volumes[]
 	uint8_t buffer[4096];
 	struct spa_pod_builder b = { 0 };
 	struct spa_pod_frame f[1];
+	uint32_t i;
+	int changed = 0;
+
+	if (n_volumes == 0)
+		return -EINVAL;
 
 	spa_log_info(this->log, "node %p volume %f", node, volumes[0]);
 
-	node->n_channels = n_volumes;
-	memcpy(node->volumes, volumes, sizeof(float) * SPA_AUDIO_MAX_CHANNELS);
+	for (i = 0; i < node->n_channels; i++) {
+		if (node->volumes[i] != volumes[i % n_volumes])
+			++changed;
+		node->volumes[i] = volumes[i % n_volumes];
+	}
 
 	spa_pod_builder_init(&b, buffer, sizeof(buffer));
 	spa_pod_builder_push_object(&b, &f[0],
@@ -932,14 +941,14 @@ static int node_set_volume(struct impl *this, struct node *node, float volumes[]
 	spa_pod_builder_add_object(&b,
 			SPA_TYPE_OBJECT_Props, SPA_EVENT_DEVICE_Props,
 			SPA_PROP_channelVolumes, SPA_POD_Array(sizeof(float),
-				SPA_TYPE_Float, n_volumes, volumes),
+				SPA_TYPE_Float, node->n_channels, node->volumes),
 			SPA_PROP_channelMap, SPA_POD_Array(sizeof(uint32_t),
 				SPA_TYPE_Id, node->n_channels, node->channels));
 	event = spa_pod_builder_pop(&b, &f[0]);
 
 	spa_device_emit_event(&this->hooks, event);
 
-	return 0;
+	return changed;
 }
 
 static int node_set_mute(struct impl *this, struct node *node, bool mute)
@@ -948,8 +957,11 @@ static int node_set_mute(struct impl *this, struct node *node, bool mute)
 	uint8_t buffer[4096];
 	struct spa_pod_builder b = { 0 };
 	struct spa_pod_frame f[1];
+	int changed = 0;
 
 	spa_log_info(this->log, "node %p mute %d", node, mute);
+
+	changed = (node->mute != mute);
 	node->mute = mute;
 
 	spa_pod_builder_init(&b, buffer, sizeof(buffer));
@@ -966,7 +978,37 @@ static int node_set_mute(struct impl *this, struct node *node, bool mute)
 
 	spa_device_emit_event(&this->hooks, event);
 
-	return 0;
+	return changed;
+}
+
+static int node_set_latency_offset(struct impl *this, struct node *node, int64_t latency_offset)
+{
+	struct spa_event *event;
+	uint8_t buffer[4096];
+	struct spa_pod_builder b = { 0 };
+	struct spa_pod_frame f[1];
+	int changed = 0;
+
+	spa_log_info(this->log, "node %p latency offset %"PRIi64" nsec", node, latency_offset);
+
+	changed = (node->latency_offset != latency_offset);
+	node->latency_offset = latency_offset;
+
+	spa_pod_builder_init(&b, buffer, sizeof(buffer));
+	spa_pod_builder_push_object(&b, &f[0],
+			SPA_TYPE_EVENT_Device, SPA_DEVICE_EVENT_ObjectConfig);
+	spa_pod_builder_prop(&b, SPA_EVENT_DEVICE_Object, 0);
+	spa_pod_builder_int(&b, node->id);
+	spa_pod_builder_prop(&b, SPA_EVENT_DEVICE_Props, 0);
+
+	spa_pod_builder_add_object(&b,
+			SPA_TYPE_OBJECT_Props, SPA_EVENT_DEVICE_Props,
+			SPA_PROP_latencyOffsetNsec, SPA_POD_Long(latency_offset));
+	event = spa_pod_builder_pop(&b, &f[0]);
+
+	spa_device_emit_event(&this->hooks, event);
+
+	return changed;
 }
 
 static int apply_device_props(struct impl *this, struct node *node, struct spa_pod *props)
@@ -978,7 +1020,8 @@ static int apply_device_props(struct impl *this, struct node *node, struct spa_p
 	int changed = 0;
 	float volumes[SPA_AUDIO_MAX_CHANNELS];
 	uint32_t channels[SPA_AUDIO_MAX_CHANNELS];
-	uint32_t n_volumes = 0, n_channels = 0;
+	uint32_t n_volumes = 0, SPA_UNUSED n_channels = 0;
+	int64_t latency_offset = 0;
 
 	if (!spa_pod_is_object_type(props, SPA_TYPE_OBJECT_Props))
 		return -EINVAL;
@@ -987,32 +1030,39 @@ static int apply_device_props(struct impl *this, struct node *node, struct spa_p
 		switch (prop->key) {
 		case SPA_PROP_volume:
 			if (spa_pod_get_float(&prop->value, &volume) == 0) {
-				node_set_volume(this, node, &volume, 1);
-				changed++;
+				int res = node_set_volume(this, node, &volume, 1);
+				if (res > 0)
+					++changed;
 			}
 			break;
 		case SPA_PROP_mute:
 			if (spa_pod_get_bool(&prop->value, &mute) == 0) {
-				node_set_mute(this, node, mute);
-				changed++;
+				int res = node_set_mute(this, node, mute);
+				if (res > 0)
+					++changed;
 			}
 			break;
 		case SPA_PROP_channelVolumes:
-			if ((n_volumes = spa_pod_copy_array(&prop->value, SPA_TYPE_Float,
-					volumes, SPA_AUDIO_MAX_CHANNELS)) > 0) {
-				changed++;
-			}
+			n_volumes = spa_pod_copy_array(&prop->value, SPA_TYPE_Float,
+					volumes, SPA_AUDIO_MAX_CHANNELS);
 			break;
 		case SPA_PROP_channelMap:
-			if ((n_channels = spa_pod_copy_array(&prop->value, SPA_TYPE_Id,
-					channels, SPA_AUDIO_MAX_CHANNELS)) > 0) {
-				changed++;
-			}
+			n_channels = spa_pod_copy_array(&prop->value, SPA_TYPE_Id,
+					channels, SPA_AUDIO_MAX_CHANNELS);
 			break;
+		case SPA_PROP_latencyOffsetNsec:
+			if (spa_pod_get_long(&prop->value, &latency_offset) == 0) {
+				int res = node_set_latency_offset(this, node, latency_offset);
+				if (res > 0)
+					++changed;
+			}
 		}
 	}
-	if (n_volumes > 0)
-		node_set_volume(this, node, volumes, n_volumes);
+	if (n_volumes > 0) {
+		int res = node_set_volume(this, node, volumes, n_volumes);
+		if (res > 0)
+			++changed;
+	}
 
 	return changed;
 }
@@ -1070,9 +1120,11 @@ static int impl_set_param(void *object,
 
 		node = &this->nodes[device];
 		if (props) {
-			apply_device_props(this, node, props);
-			this->info.change_mask |= SPA_DEVICE_CHANGE_MASK_PARAMS;
-			this->params[IDX_Route].flags ^= SPA_PARAM_INFO_SERIAL;
+			int changed = apply_device_props(this, node, props);
+			if (changed > 0) {
+				this->info.change_mask |= SPA_DEVICE_CHANGE_MASK_PARAMS;
+				this->params[IDX_Route].flags ^= SPA_PARAM_INFO_SERIAL;
+			}
 			emit_info(this, false);
 		}
 		break;
