@@ -29,8 +29,10 @@
 extern "C" {
 #endif
 
-#include <spa/support/plugin.h>
+#include <spa/support/dbus.h>
+#include <spa/support/log.h>
 #include <spa/support/loop.h>
+#include <spa/support/plugin.h>
 #include <spa/utils/hook.h>
 
 #include "config.h"
@@ -143,6 +145,8 @@ extern "C" {
 #define A2DP_SINK_ENDPOINT	A2DP_OBJECT_MANAGER_PATH "/A2DPSink"
 #define A2DP_SOURCE_ENDPOINT	A2DP_OBJECT_MANAGER_PATH "/A2DPSource"
 
+#define SPA_BT_UNKNOWN_DELAY			0
+
 /* HFP uses SBC encoding with precisely defined parameters. Hence, the size
  * of the input (number of PCM samples) and output is known up front. */
 #define MSBC_DECODED_SIZE       240
@@ -386,7 +390,6 @@ struct spa_bt_device {
 	struct spa_list remote_endpoint_list;
 	struct spa_list transport_list;
 	struct spa_list codec_switch_list;
-	struct spa_bt_a2dp_codec_switch *active_codec_switch;
 
 	struct spa_hook_list listener_list;
 	bool added;
@@ -481,6 +484,7 @@ struct spa_bt_transport *spa_bt_transport_find(struct spa_bt_monitor *monitor, c
 struct spa_bt_transport *spa_bt_transport_find_full(struct spa_bt_monitor *monitor,
                                                     bool (*callback) (struct spa_bt_transport *t, const void *data),
                                                     const void *data);
+int64_t spa_bt_transport_get_delay_nsec(struct spa_bt_transport *t);
 
 int spa_bt_transport_acquire(struct spa_bt_transport *t, bool optional);
 int spa_bt_transport_release(struct spa_bt_transport *t);
@@ -530,6 +534,7 @@ struct spa_bt_backend *backend_native_new(struct spa_bt_monitor *monitor,
 		uint32_t n_support);
 void backend_native_free(struct spa_bt_backend *backend);
 void backend_native_register_profiles(struct spa_bt_backend *backend);
+void backend_native_unregister_profiles(struct spa_bt_backend *backend);
 #else
 static inline struct spa_bt_backend *backend_native_new(struct spa_bt_monitor *monitor,
 		void *dbus_connection,
@@ -540,8 +545,10 @@ static inline struct spa_bt_backend *backend_native_new(struct spa_bt_monitor *m
 }
 static inline void backend_native_free(struct spa_bt_backend *backend) {}
 static inline void backend_native_register_profiles(struct spa_bt_backend *backend) {}
+static inline void backend_native_unregister_profiles(struct spa_bt_backend *backend) {}
 #endif
 
+#define OFONO_SERVICE "org.ofono"
 #ifdef HAVE_BLUEZ_5_BACKEND_OFONO
 struct spa_bt_backend *backend_ofono_new(struct spa_bt_monitor *monitor,
 		void *dbus_connection,
@@ -549,6 +556,7 @@ struct spa_bt_backend *backend_ofono_new(struct spa_bt_monitor *monitor,
 		const struct spa_support *support,
 		uint32_t n_support);
 void backend_ofono_free(struct spa_bt_backend *backend);
+int backend_ofono_register(struct spa_bt_backend *backend);
 void backend_ofono_add_filters(struct spa_bt_backend *backend);
 #else
 static inline struct spa_bt_backend *backend_ofono_new(struct spa_bt_monitor *monitor,
@@ -559,9 +567,11 @@ static inline struct spa_bt_backend *backend_ofono_new(struct spa_bt_monitor *mo
 	return NULL;
 }
 static inline void backend_ofono_free(struct spa_bt_backend *backend) {}
+static inline int backend_ofono_register(struct spa_bt_backend *backend) { return -ENOTSUP; }
 static inline void backend_ofono_add_filters(struct spa_bt_backend *backend) {}
 #endif
 
+#define HSPHFPD_SERVICE "org.hsphfpd"
 #ifdef HAVE_BLUEZ_5_BACKEND_HSPHFPD
 struct spa_bt_backend *backend_hsphfpd_new(struct spa_bt_monitor *monitor,
 		void *dbus_connection,
@@ -569,6 +579,8 @@ struct spa_bt_backend *backend_hsphfpd_new(struct spa_bt_monitor *monitor,
 		const struct spa_support *support,
 		uint32_t n_support);
 void backend_hsphfpd_free(struct spa_bt_backend *backend);
+int backend_hsphfpd_register(struct spa_bt_backend *backend);
+void backend_hsphfpd_unregistered(struct spa_bt_backend *backend);
 void backend_hsphfpd_add_filters(struct spa_bt_backend *backend);
 #else
 static inline struct spa_bt_backend *backend_hsphfpd_new(struct spa_bt_monitor *monitor,
@@ -579,6 +591,8 @@ static inline struct spa_bt_backend *backend_hsphfpd_new(struct spa_bt_monitor *
 	return NULL;
 }
 static inline void backend_hsphfpd_free(struct spa_bt_backend *backend) {}
+static inline int backend_hsphfpd_register(struct spa_bt_backend *backend) { return -ENOTSUP; }
+static inline void backend_hsphfpd_unregistered(struct spa_bt_backend *backend) {}
 static inline void backend_hsphfpd_add_filters(struct spa_bt_backend *backend) {}
 #endif
 
