@@ -657,6 +657,8 @@ struct pw_impl_node {
 
 	struct spa_fraction latency;		/**< requested latency */
 	uint32_t quantum_size;			/**< desired quantum */
+	struct spa_fraction max_latency;	/**< miximum latency */
+	uint32_t max_quantum_size;		/**< max supported quantum */
 	struct spa_source source;		/**< source to remotely trigger this node */
 	struct pw_memblock *activation;
 	struct {
