@@ -33,9 +33,10 @@
 #endif
 #include <getopt.h>
 
+#define spa_debug(...) fprintf(stdout,__VA_ARGS__);fputc('\n', stdout)
+
 #include <spa/utils/result.h>
 #include <spa/debug/pod.h>
-#include <spa/debug/format.h>
 #include <spa/utils/keys.h>
 #include <spa/utils/json.h>
 #include <spa/pod/builder.h>
@@ -436,7 +437,6 @@ static void on_core_destroy(void *_data)
 	spa_list_remove(&rd->link);
 
 	spa_hook_remove(&rd->core_listener);
-	spa_hook_remove(&rd->registry_listener);
 	spa_hook_remove(&rd->proxy_core_listener);
 
 	pw_map_remove(&data->vars, rd->id);
@@ -455,6 +455,7 @@ static const struct pw_proxy_events proxy_core_events = {
 
 static void remote_data_free(struct remote_data *rd)
 {
+	spa_hook_remove(&rd->registry_listener);
 	pw_proxy_destroy((struct pw_proxy*)rd->registry);
 	pw_core_disconnect(rd->core);
 }
@@ -681,7 +682,7 @@ static void info_link(struct proxy_data *pd)
 		printf("\n");
 	fprintf(stdout, "%c\tformat:\n", MARK_CHANGE(PW_LINK_CHANGE_MASK_FORMAT));
 	if (info->format)
-		spa_debug_format(2, NULL, info->format);
+		spa_debug_pod(2, NULL, info->format);
 	else
 		fprintf(stdout, "\t\tnone\n");
 	print_properties(info->props, MARK_CHANGE(PW_LINK_CHANGE_MASK_PROPS), true);
@@ -815,10 +816,7 @@ static void event_param(void *object, int seq, uint32_t id,
 		fprintf(stdout, "remote %d object %d param %d index %d\n",
 				rd->id, data->global->id, id, index);
 
-	if (spa_pod_is_object_type(param, SPA_TYPE_OBJECT_Format))
-		spa_debug_format(2, NULL, param);
-	else
-		spa_debug_pod(2, NULL, param);
+	spa_debug_pod(2, NULL, param);
 }
 
 static const struct pw_node_events node_events = {
@@ -2567,7 +2565,7 @@ dump_link(struct data *data, struct global *global,
 			printf("\n");
 		fprintf(stdout, "%sformat:\n", ind);
 		if (info->format)
-			spa_debug_format(8 * (level + 1) + 2, NULL, info->format);
+			spa_debug_pod(8 * (level + 1) + 2, NULL, info->format);
 		else
 			fprintf(stdout, "%s\tnone\n", ind);
 
@@ -3004,6 +3002,10 @@ int main(int argc, char *argv[])
 	}
 
 	data.loop = pw_main_loop_new(NULL);
+	if (data.loop == NULL) {
+		fprintf(stderr, "Broken installation: %m\n");
+		return -1;
+	}
 	l = pw_main_loop_get_loop(data.loop);
 	pw_loop_add_signal(l, SIGINT, do_quit, &data);
 	pw_loop_add_signal(l, SIGTERM, do_quit, &data);
